"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DataProvider_1 = require("./DataProvider");
var DataProviderBuilder = /** @class */ (function () {
    function DataProviderBuilder(fetcher) {
        this._initialValue = undefined;
        this._fetchRetention = 0;
        if (!fetcher) {
            throw new Error("invalid fetcher");
        }
        this._fetcher = fetcher;
        return this;
    }
    DataProviderBuilder.prototype.setInitialValue = function (f) {
        this._initialValue = f();
        return this;
    };
    DataProviderBuilder.prototype.setFetchRetention = function (retention) {
        this._fetchRetention = retention;
        return this;
    };
    DataProviderBuilder.prototype.useCache = function (cache, cacheKey) {
        if (!cache) {
            throw new Error("invalid cache");
        }
        if (!cacheKey) {
            throw new Error("invalid cacheKey");
        }
        this._cache = cache;
        this._cacheKey = cacheKey;
        return this;
    };
    DataProviderBuilder.prototype.usePostFetch = function (f) {
        this._postFetch = f;
        return this;
    };
    DataProviderBuilder.prototype.build = function () {
        return new DataProvider_1.DataProvider(this._initialValue, this._fetcher, this._fetchRetention, this._postFetch, this._cache, this._cacheKey);
    };
    return DataProviderBuilder;
}());
exports.DataProviderBuilder = DataProviderBuilder;
//# sourceMappingURL=DataProviderBuilder.js.map