var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
export class DataProvider {
    constructor(initialValue, fetcher, fetchRetention = 0, postFetch, cache, cacheKey) {
        this._fetchRetention = 0;
        this._postFetch = undefined;
        this._cache = undefined;
        this._cacheKey = "";
        this._subscriptions = new Array();
        this._lastFetched = 0;
        this._loadedFromCache = false;
        this._data = undefined;
        // private _dataLoaded: boolean = false;
        this._fetching = false;
        this._fetchPromise = undefined;
        this._data = initialValue;
        this._fetcher = fetcher;
        this._fetchRetention = fetchRetention > 0 ? fetchRetention : 0;
        this._postFetch = postFetch;
        if (cache) {
            this._cache = cache;
            if (!cacheKey) {
                throw new Error("invalid cacheKey");
            }
            else {
                this._cacheKey = DataProvider._prefix + cacheKey;
            }
        }
    }
    subscribe(subscription) {
        const handle = this._subscriptions.push(subscription) - 1;
        if (this._data === undefined) {
            this.loadData();
        }
        else {
            this.invokeDataChanged(subscription);
        }
        return handle;
    }
    unsubscribe(handle) {
        if (handle < 0 || handle >= this._subscriptions.length) {
            throw new Error(`index out of bound ${handle}/${this._subscriptions.length}`);
        }
        this._subscriptions[handle] = undefined;
    }
    getLocal(triggerFetch = true) {
        return __awaiter(this, void 0, void 0, function* () {
            if (triggerFetch) {
                this.fetchRemote();
            }
            if (this._cache !== undefined && !this._loadedFromCache) {
                let data = yield this._cache.get(this._cacheKey);
                this._loadedFromCache = true;
                this._data = data;
                this.fireDataChangedEvent(data);
            }
            return this._data;
        });
    }
    setLocal(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this._data = data;
            if (this._cache) {
                yield this._cache.set(this._cacheKey, data);
            }
            this.fireDataChangedEvent(data);
        });
    }
    fetchRemote() {
        return __awaiter(this, void 0, void 0, function* () {
            if (Date.now() - this._lastFetched < this._fetchRetention) {
                return this._data;
            }
            if (this._fetching && this._fetchPromise) {
                return this._fetchPromise;
            }
            this._fetchPromise = this.innerFetchRemote();
            return this._fetchPromise;
        });
    }
    innerFetchRemote() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this._fetching = true;
                // fire onFetching
                this.fireFetchingEvent();
                let data = yield this._fetcher();
                this._lastFetched = Date.now();
                this._fetching = false;
                if (this._postFetch) {
                    data = yield this._postFetch(this._data, data);
                }
                this._data = data;
                // set local
                if (this._cache !== undefined) {
                    yield this._cache.set(this._cacheKey, this._data);
                }
                this.fireFetchedEvent();
                return this._data;
            }
            catch (err) {
                this._fetching = false;
                this.fireFetchErrorEvent(err);
                throw err;
            }
        });
    }
    loadData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._cache !== undefined && !this._loadedFromCache) {
                let data = yield this._cache.get(this._cacheKey);
                this._loadedFromCache = true;
                if (data !== undefined) {
                    this.fireDataChangedEvent(data);
                }
                else {
                    yield this.fetchRemote();
                }
            }
            else {
                yield this.fetchRemote();
            }
        });
    }
    invokeDataChanged(sub) {
        if (this._data === undefined) {
            return;
        }
        else {
            try {
                sub.dataHandler && sub.dataHandler(this._data);
            }
            catch (err) {
                sub.onError && sub.onError(err);
            }
        }
    }
    fireDataChangedEvent(data) {
        if (data === undefined) {
            return;
        }
        else {
            this._data = data;
            this._subscriptions.forEach(sub => {
                sub && this.invokeDataChanged(sub);
            });
        }
    }
    invokeFetching(sub) {
        try {
            sub.onFetching && sub.onFetching();
        }
        catch (err) {
            sub.onError && sub.onError(err);
        }
    }
    fireFetchingEvent() {
        this._subscriptions.forEach(sub => {
            sub && this.invokeFetching(sub);
        });
    }
    invokeFetched(sub) {
        if (this._data === undefined) {
            return;
        }
        try {
            sub.onFetched && sub.onFetched(this._data);
        }
        catch (err) {
            sub.onError && sub.onError(err);
        }
    }
    fireFetchedEvent() {
        this._subscriptions.forEach(sub => {
            if (!sub) {
                return;
            }
            if (sub.onFetched) {
                this.invokeFetched(sub);
            }
            else {
                this.invokeDataChanged(sub);
            }
        });
    }
    invokeFetchError(sub, err) {
        try {
            sub.onFetchError && sub.onFetchError(err);
        }
        catch (_a) { }
    }
    fireFetchErrorEvent(err) {
        this._subscriptions.forEach(sub => {
            sub && this.invokeFetchError(sub, err);
        });
    }
}
DataProvider._prefix = "$data:";
//# sourceMappingURL=DataProvider.js.map