import { DataProvider } from "./DataProvider";
export class DataProviderBuilder {
    constructor(fetcher) {
        this._initialValue = undefined;
        this._fetchRetention = 0;
        if (!fetcher) {
            throw new Error("invalid fetcher");
        }
        this._fetcher = fetcher;
        return this;
    }
    setInitialValue(f) {
        this._initialValue = f();
        return this;
    }
    setFetchRetention(retention) {
        this._fetchRetention = retention;
        return this;
    }
    useCache(cache, cacheKey) {
        if (!cache) {
            throw new Error("invalid cache");
        }
        if (!cacheKey) {
            throw new Error("invalid cacheKey");
        }
        this._cache = cache;
        this._cacheKey = cacheKey;
        return this;
    }
    usePostFetch(f) {
        this._postFetch = f;
        return this;
    }
    build() {
        return new DataProvider(this._initialValue, this._fetcher, this._fetchRetention, this._postFetch, this._cache, this._cacheKey);
    }
}
//# sourceMappingURL=DataProviderBuilder.js.map